/*
    ITmages  upload client for Windows
    Copyright (C) 2011 Dmitriy Simbiriatin <slpiv@mail.ru>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <windows.h>
#include <wchar.h>
#include <wctype.h>
#include <FreeImage.h>
#include "imagefunc.h"
#include "enconv.h"
#include "errorshandler.h"

void SetExtToLower(wchar_t *ext)
{
    size_t i;

    for (i = 0; i < wcslen(ext); ++i) {
        ext[i] = towlower(ext[i]);
    }
}

size_t GetFilterType(size_t filter_index)
{
    size_t filter_type;

    switch (filter_index) {
    case 0:
        filter_type = FILTER_LANCZOS3; // Lanczos filter
        break;
    case 1:
        filter_type = FILTER_BOX; // Box filter
        break;
    case 2:
        filter_type = FILTER_BILINEAR; // Bilinear filter
        break;
    case 3:
        filter_type = FILTER_BSPLINE; // B-Spline filter
        break;
    }
    return filter_type;
}

int GetImageInfo(
        wchar_t *image_path,
        wchar_t *basename,
        wchar_t *ext,
        ItmagesImageInfo *imageInfo)
{
    FIBITMAP *image = NULL;

    size_t image_type = FreeImage_GetFileTypeU(image_path, 0);

    image = FreeImage_LoadU(image_type, image_path, 0);
    if (image == NULL) {
        return -1;
    }
    
    imageInfo->mime = (char*) FreeImage_GetFIFMimeType(image_type);
    imageInfo->fullname = GetImageFullname(basename, ext);
    imageInfo->width = FreeImage_GetWidth(image);
    imageInfo->height = FreeImage_GetHeight(image);
    imageInfo->size = GetImageSize(image_path);

    if (image) {
        FreeImage_Unload(image);
    }
    return 0;
}

void FreeImageInfo(ItmagesImageInfo *imageInfo)
{
    if (imageInfo->fullname != NULL) {
        free(imageInfo->fullname);
        imageInfo->fullname = NULL;
    }
    if (imageInfo->mime != NULL) {
        imageInfo->mime = NULL;
    }
    imageInfo->width = 0;
    imageInfo->height = 0;
    imageInfo->size = 0;
}

int ResizeImageTo(
        wchar_t *image_path,
        size_t new_width,
        size_t filter_index)
{
    FIBITMAP *image = NULL;
    FIBITMAP *resized_image = NULL;

    float width, height, ratio;
    size_t new_height;

    size_t image_type = FreeImage_GetFileTypeU(image_path, 0);

    image = FreeImage_LoadU(image_type, image_path, 0);
    if (image == NULL) {
        return -1;
    }
    width = FreeImage_GetWidth(image);
    height = FreeImage_GetHeight(image);

    ratio = width / new_width;
    new_height = height / ratio;
	
	if (new_width < 1) {
		new_width = 1;
	}
    if (new_height < 1) {
        new_height = 1;
    }

    if (new_height > height) {
        if (image) FreeImage_Unload(image);
        return -1;
    }

    size_t filter_type = GetFilterType(filter_index);
    resized_image = FreeImage_Rescale(image, new_width, new_height, filter_type);
    if (resized_image == NULL) {
        if (image) FreeImage_Unload(image);
        return -1;
    }
    FreeImage_SaveU(image_type, resized_image, image_path, 0);

    if (resized_image) {
        FreeImage_Unload(resized_image);
    }

    if (image) {
        FreeImage_Unload(image);
    }

    return 0;
}

int RotateImageTo(wchar_t *image_path, int angle)
{
    FIBITMAP *image = NULL;
    FIBITMAP *rotated_image = NULL;

    size_t image_type = FreeImage_GetFileTypeU(image_path, 0);

    image = FreeImage_LoadU(image_type, image_path, 0);
    if (image == NULL) {
        return -1;
    }

    rotated_image = FreeImage_Rotate(image, angle, NULL);
    if (rotated_image == NULL) {
        if (image) FreeImage_Unload(image);
        return -1;
    }
    FreeImage_SaveU(image_type, rotated_image, image_path, 0);

    if (rotated_image) {
        FreeImage_Unload(rotated_image);
    }

    if (image) {
        FreeImage_Unload(image);
    }

    return 0;
}

HBITMAP HBITMAPFromFile(
        HWND image_wnd,
        wchar_t *image_path,
        size_t max_width,
        size_t max_height)
{
    FIBITMAP *image = NULL;
    FIBITMAP *resized_image = NULL;

    size_t width, height, new_width, new_height;

    size_t image_type = FreeImage_GetFileTypeU(image_path, 0);

    image = FreeImage_LoadU(image_type, image_path, 0);
    if (image == NULL) {
        return NULL;
    }
	
    width = FreeImage_GetWidth(image);
    height = FreeImage_GetHeight(image);

    if (width <= max_width) {
        new_width = width;
    } else {
        new_width = max_width;
    }
    if (height <= max_height) {
        new_height = height;
    } else {
        new_height = max_height;
    }

    resized_image = FreeImage_Rescale(image, new_width, new_height, FILTER_BOX);
    if (resized_image == NULL) {
        if (image) FreeImage_Unload(image);
        return NULL;
    }

    HBITMAP bitmap;
    HDC image_dc;

    image_dc = GetDC(image_wnd);
    bitmap = CreateDIBitmap(
            image_dc,
            FreeImage_GetInfoHeader(resized_image),
            CBM_INIT,
            FreeImage_GetBits(resized_image),
            FreeImage_GetInfo(resized_image),
            DIB_RGB_COLORS);
    DeleteDC(image_dc);
    ReleaseDC(image_wnd, image_dc);

    if (resized_image) {
        FreeImage_Unload(resized_image);
    }

    if (image) {
        FreeImage_Unload(image);
    }

    return bitmap;
}

int CheckImageExtension(wchar_t *ext)
{
    if (
            wmemcmp(ext, L".jpg", 5) != 0 &&
            wmemcmp(ext, L".jpeg", 6) != 0 &&
            wmemcmp(ext, L".png", 5) != 0 &&
            wmemcmp(ext, L".gif", 5) != 0
            ) {
        return -1;
    }
    return 0;
}

int GetImageSize(wchar_t *image_path)
{
    struct _stat image;

    _wstat(image_path, &image);
    if (image.st_size > MAX_IMAGE_SIZE) {
        return -1;
    } else {
        return image.st_size;
    }
}

char *GetImageFullname(wchar_t *basename, wchar_t *ext)
{
    size_t base_len = wcslen(basename);
    size_t ext_len = wcslen(ext);

    wchar_t *fullname = (wchar_t*) malloc((base_len + ext_len + 1) * sizeof (wchar_t));
    if (fullname == NULL) {
        ErrorsHandler(ERROR_MEMALLOC);
        exit(EXIT_FAILURE);
    }

    wmemcpy(fullname, basename, base_len + 1);
    wmemcpy(fullname + base_len, ext, ext_len + 1);

    char *fullname_mb = WideToUtf8(fullname);
    free(fullname);
	
    return fullname_mb;
}