/*
    ITmages  upload client for Windows
    Copyright (C) 2011 Dmitriy Simbiriatin <slpiv@itmages.ru>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#define _WIN32_WINNT 0x0501
#define _WIN32_IE 0x0501
#define WINVER 0x0501

#include <windows.h>
#include <wchar.h>
#include <shlobj.h>
#include <uxtheme.h>
#include <process.h>
#include <libintl.h>
#include <locale.h>
#include "itmagesgui.h"
#include "itmagescmd.h"
#include "imagefunc.h"
#include "errorshandler.h"
#include "enconv.h"

// Window classes --------------------------------------------------------------
const char g_LoginWindowClass[] = "ITmagesLogin";
const char g_AboutWindowClass[] = "ITmagesAbout";
const char g_ReadyWindowClass[] = "ITmagesReady";
// -----------------------------------------------------------------------------

// Main windows ----------------------------------------------------------------
HWND login_wnd = NULL;
HWND ready_wnd = NULL;
HWND about_wnd = NULL;
// -----------------------------------------------------------------------------

HINSTANCE g_ItmagesInst = NULL;

// Default system's font.
HFONT g_SystemFont = NULL;

// Application's icon.
HICON g_AppIcon = NULL;

// Static control's background brush.
HBRUSH g_StaticBrush = NULL;

// Image's preview.
HBITMAP previewImage = NULL;

// Indicates if the new visual style is enabled or not.
short g_ThemeActive = 0;

ItmagesImageInfo imageInfo;
ItmagesImageLinks imageLinks;
ItmagesImageData imageData;
ItmagesUserInfo userInfo;
ItmagesConfig configFiles;

INT APIENTRY WinMain(
        HINSTANCE inst,
        HINSTANCE prev_inst,
        LPSTR cmd_line,
        int cmd_show)
{
    g_ItmagesInst = inst;

    // Loading application's icon.
    g_AppIcon = LoadIcon(g_ItmagesInst, MAKEINTRESOURCE(ID_APPICON));

    // Loading default system's font.
    g_SystemFont = (HFONT) GetStockObject(DEFAULT_GUI_FONT);

    // Getting directory with the localization files and choose an appropriate one.
    char *locale_dir = GetLocaleDir();
    if (locale_dir != NULL) {
        setlocale(LC_ALL, "");
        bindtextdomain("itmages", locale_dir);
        textdomain("itmages");
    }
    free(locale_dir);

    LPWSTR *arg_list;
    int arg_count;

    // Retrieving a path to the image from the command line.
    arg_list = CommandLineToArgvW(GetCommandLineW(), &arg_count);
    if (arg_list == NULL) {
        ErrorsHandler(ERROR_GETIMGPATH);
        exit(EXIT_FAILURE);
    }

    if (arg_count == 1) {
        // If the path was not provided, loading the open file dialog.
        char *image_path = CreateOpenImageDialog();
        imageData.path = AcpToWide(image_path);
        free(image_path);
    } else {
        size_t path_len = wcslen(arg_list[1]);
        imageData.path = (wchar_t*) malloc((path_len + 1) * sizeof (wchar_t));
        if (imageData.path == NULL) {
            ErrorsHandler(ERROR_MEMALLOC);
            exit(EXIT_FAILURE);
        }
        wmemcpy(imageData.path, arg_list[1], path_len + 1);
    }
    LocalFree(arg_list);

    imageData.basename = (wchar_t*) calloc(_MAX_FNAME + 1, sizeof (wchar_t));
    if (imageData.basename == NULL) {
        ErrorsHandler(ERROR_MEMALLOC);
        exit(EXIT_FAILURE);
    }
    imageData.ext = (wchar_t*) calloc(_MAX_EXT + 1, sizeof (wchar_t));
    if (imageData.ext == NULL) {
        ErrorsHandler(ERROR_MEMALLOC);
        exit(EXIT_FAILURE);
    }
    // Retrieving the image's basename and extension from the image's path.
    _wsplitpath(imageData.path, NULL, NULL, imageData.basename, imageData.ext);

    // Retrieving an information about the image(e.g name, width, height, size etc.)
    if (GetImageInfo(imageData.path, imageData.basename,
				imageData.ext, &imageInfo) == -1) {
        ErrorsHandler(ERROR_GETIMGINFO);
        exit(EXIT_FAILURE);
    }

    // Setting the image's extension to the lower case
    SetExtToLower(imageData.ext);

    /* Checking the image's extension. ITmages supports only 
     *.jpg, *.jpeg, *.png, *.gif images. */
    if (CheckImageExtension(imageData.ext) == -1) {
        ErrorsHandler(ERROR_SUPPORTEDIMG);
        exit(EXIT_FAILURE);
    }

    // Checking the image's size. The size of the image has to be less than 6 Mb.
    if (imageInfo.size == -1) {
        ErrorsHandler(ERROR_MAXIMGSIZE);
        exit(EXIT_FAILURE);
    } else if (imageInfo.size == 0) {
        ErrorsHandler(ERROR_BADIMGSIZE);
        exit(EXIT_FAILURE);
    }

    // Retrieving the system's default path for the config and cookies files.
    GetConfigFiles(&configFiles);

    // Registering window classes.
    if (
            WinLogin() == -1 ||
            WinAbout() == -1 ||
            WinReady() == -1
            ) {
        ErrorsHandler(ERROR_REGISTERWNDCLS);
    }

    CreateLoginWnd();

    MSG msg;

    while (GetMessage(&msg, NULL, 0, 0)) {
        if (
                !IsDialogMessage(login_wnd, &msg) &&
                !IsDialogMessage(about_wnd, &msg) &&
                !IsDialogMessage(ready_wnd, &msg)
                ) {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }
    return (DWORD) msg.wParam;
}

INT APIENTRY WinLogin()
{
    WNDCLASSEX itmagesLogin;
    memset(&itmagesLogin, 0, sizeof (itmagesLogin));

    itmagesLogin.cbSize = sizeof (itmagesLogin);
    itmagesLogin.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
    itmagesLogin.lpfnWndProc = WinMainProc;
    itmagesLogin.cbClsExtra = 0;
    itmagesLogin.cbWndExtra = 0;
    itmagesLogin.hInstance = g_ItmagesInst;
    itmagesLogin.hIcon = g_AppIcon;
    itmagesLogin.hCursor = LoadCursor(NULL, IDC_ARROW);
    itmagesLogin.hbrBackground = (HBRUSH) (COLOR_BTNFACE + 1);
    itmagesLogin.lpszMenuName = NULL;
    itmagesLogin.lpszClassName = g_LoginWindowClass;
    itmagesLogin.hIconSm = NULL;

    if (RegisterClassEx(&itmagesLogin) == 0) {
        return -1;
    }
    return 0;
}

void CreateLoginWnd()
{
    size_t desk_width = GetSystemMetrics(SM_CXSCREEN);
    size_t desk_height = GetSystemMetrics(SM_CYSCREEN);

    login_wnd = CreateWindowEx(
            0,
            g_LoginWindowClass,
            gettext("ITmages - Login"),
            WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX | WS_CLIPCHILDREN,
            desk_width / 2 - LOGIN_WND_WIDTH / 2,
            desk_height / 2 - LOGIN_WND_HEIGHT / 2,
            LOGIN_WND_WIDTH,
            LOGIN_WND_HEIGHT,
            NULL,
            NULL,
            g_ItmagesInst,
            NULL);

    // Disabling maximize button.
    long wnd_style = GetWindowLong(login_wnd, GWL_STYLE);
    SetWindowLong(login_wnd, GWL_STYLE, wnd_style & ~WS_MAXIMIZEBOX);

    // Creating Tab control inside the Login window.
    CreateLoginTabControl();

    ShowWindow(login_wnd, SW_SHOW);
    UpdateWindow(login_wnd);
}

void CreateLoginTabControl()
{
    HWND login_tab = CreateWindowEx(
            0,
            WC_TABCONTROL,
            "",
            WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_CLIPCHILDREN,
            LOGIN_TABCONTROL_X,
            LOGIN_TABCONTROL_Y,
            LOGIN_TABCONTROL_WIDTH,
            LOGIN_TABCONTROL_HEIGHT,
            login_wnd,
            (HMENU) ID_LOGIN_TABCONTROL,
            g_ItmagesInst,
            NULL
            );

    SendMessage(login_tab, WM_SETFONT, (WPARAM) g_SystemFont, 0);

    TCITEM tie;
    memset(&tie, 0, sizeof (tie));

    tie.mask = TCIF_TEXT;
    tie.pszText = gettext("Account");
    TabCtrl_InsertItem(login_tab, 0, &tie);
    tie.pszText = gettext("Image settings");
    TabCtrl_InsertItem(login_tab, 1, &tie);
    tie.pszText = gettext("Proxy");
    TabCtrl_InsertItem(login_tab, 2, &tie);

    UpdateBackgroundBrush(login_tab);
}

LRESULT CALLBACK WinMainProc(
        HWND login_wnd,
        UINT message,
        WPARAM wParam,
        LPARAM lParam)
{
    HWND login_tab = GetDlgItem(login_wnd, ID_LOGIN_TABCONTROL);
    static HWND login, login_txt,
            passwd, passwd_txt,
            without_login, save_data,
            resize_img, resize_size,
            resize_updown, resize_filter,
            rotate_img, rotate_angle,
            use_proxy, proxy_addrss,
            proxy_addrss_txt, proxy_port, proxy_port_txt,
            about, upload, upload_progress,
            login_tool, passwd_tool;

    static HWND withoutlogin_tool,
            savedata_tool,
            resizeimg_tool,
            size_tool,
            filter_tool,
            addrss_tool,
            port_tool;
    switch (message) {
    case WM_CREATE:
    {
        login = CreateWindowEx(
                WS_EX_CLIENTEDGE,
                "EDIT",
                "",
                WS_CHILD | WS_VISIBLE | ES_AUTOHSCROLL | WS_TABSTOP | WS_CLIPSIBLINGS,
                LOGIN_EDIT_X,
                LOGIN_EDIT_Y,
                LOGIN_EDIT_WIDTH,
                LOGIN_EDIT_HEIGHT,
                login_wnd,
                (HMENU) ID_LOGIN,
                g_ItmagesInst,
                NULL);

        login_txt = CreateWindowEx(
                0,
                "STATIC",
                gettext("Login:"),
                WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS,
                LOGIN_TXT_X,
                LOGIN_TXT_Y,
                LOGIN_TXT_WIDTH,
                LOGIN_TXT_HEIGHT,
                login_wnd,
                (HMENU) ID_LOGIN_TXT,
                g_ItmagesInst,
                NULL);

        passwd = CreateWindowEx(
                WS_EX_CLIENTEDGE,
                "EDIT",
                "",
                WS_CHILD | WS_VISIBLE | ES_PASSWORD | ES_AUTOHSCROLL | WS_TABSTOP | WS_CLIPSIBLINGS,
                PASSWD_EDIT_X,
                PASSWD_EDIT_Y,
                PASSWD_EDIT_WIDTH,
                PASSWD_EDIT_HEIGHT,
                login_wnd,
                (HMENU) ID_PASSWD,
                g_ItmagesInst,
                NULL);
        passwd_txt = CreateWindowEx(
                0,
                "STATIC",
                gettext("Password:"),
                WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS,
                PASSWD_TXT_X,
                PASSWD_TXT_Y,
                PASSWD_TXT_WIDTH,
                PASSWD_TXT_HEIGHT,
                login_wnd,
                (HMENU) ID_PASSWD_TXT,
                g_ItmagesInst,
                NULL);

        without_login = CreateWindowEx(
                0,
                "BUTTON",
                gettext("Without account"),
                WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX | WS_TABSTOP | WS_CLIPSIBLINGS,
                CHECK_WITHOUTLOGIN_X,
                CHECK_WITHOUTLOGIN_Y,
                CHECK_WITHOUTLOGIN_WIDTH,
                CHECK_WITHOUTLOGIN_HEIGHT,
                login_wnd,
                (HMENU) ID_CHECK_WITHOUTLOGIN,
                g_ItmagesInst,
                NULL);
        save_data = CreateWindowEx(
                0,
                "BUTTON",
                gettext("Save account settings"),
                WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX | WS_TABSTOP | WS_CLIPSIBLINGS,
                CHECK_SAVEDATA_X,
                CHECK_SAVEDATA_Y,
                CHECK_SAVEDATA_WIDTH,
                CHECK_SAVEDATA_HEIGHT,
                login_wnd,
                (HMENU) ID_CHECK_SAVEDATA,
                g_ItmagesInst,
                NULL);
        resize_img = CreateWindowEx(
                0,
                "BUTTON",
                gettext("Reduce to:"),
                WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX | WS_TABSTOP | WS_CLIPSIBLINGS,
                CHECK_RESIZEIMG_X,
                CHECK_RESIZEIMG_Y,
                CHECK_RESIZEIMG_WIDTH,
                CHECK_RESIZEIMG_HEIGHT,
                login_wnd,
                (HMENU) ID_CHECK_RESIZEIMG,
                g_ItmagesInst,
                NULL);
        resize_size = CreateWindowEx(
                WS_EX_CLIENTEDGE,
                "EDIT",
                "",
                WS_CHILD | WS_VISIBLE | ES_LEFT | ES_NUMBER | ES_AUTOHSCROLL | WS_TABSTOP | WS_CLIPSIBLINGS,
                RESIZEIMG_SIZE_X,
                RESIZEIMG_SIZE_Y,
                RESIZEIMG_SIZE_WIDTH,
                RESIZEIMG_SIZE_HEIGHT,
                login_wnd,
                (HMENU) ID_RESIZEIMG_SIZE,
                g_ItmagesInst,
                NULL);
        resize_updown = CreateWindow(
                UPDOWN_CLASS,
                "",
                WS_CHILD | WS_VISIBLE | UDS_ARROWKEYS | UDS_ALIGNRIGHT | UDS_SETBUDDYINT | UDS_NOTHOUSANDS,
                0,
                0,
                0,
                0,
                login_wnd,
                (HMENU) ID_RESIZEIMGSIZE_UPDOWN,
                g_ItmagesInst,
                NULL);
        resize_filter = CreateWindowEx(
                0,
                "COMBOBOX",
                NULL,
                WS_CHILD | WS_VISIBLE | WS_VSCROLL | CBS_DROPDOWNLIST | CBS_HASSTRINGS | WS_TABSTOP | WS_CLIPSIBLINGS,
                RESIZE_FILTER_X,
                RESIZE_FILTER_Y,
                RESIZE_FILTER_WIDTH,
                RESIZE_FILTER_HEIGHT,
                login_wnd,
                (HMENU) ID_RESIZE_FILTER,
                g_ItmagesInst,
                NULL);
        rotate_img = CreateWindowEx(
                0,
                "BUTTON",
                gettext("Rotate:"),
                WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX | WS_TABSTOP | WS_CLIPSIBLINGS,
                CHECK_ROTATEIMG_X,
                CHECK_ROTATEIMG_Y,
                CHECK_ROTATEIMG_WIDTH,
                CHECK_ROTATEIMG_HEIGHT,
                login_wnd,
                (HMENU) ID_CHECK_ROTATEIMG,
                g_ItmagesInst,
                NULL);
        rotate_angle = CreateWindowEx(
                0,
                "COMBOBOX",
                NULL,
                WS_CHILD | WS_VISIBLE | WS_VSCROLL | CBS_DROPDOWNLIST | CBS_HASSTRINGS | WS_TABSTOP | WS_CLIPSIBLINGS,
                ROTATEIMG_ANGLE_X,
                ROTATEIMG_ANGLE_Y,
                ROTATEIMG_ANGLE_WIDTH,
                ROTATEIMG_ANGLE_HEIGHT,
                login_wnd,
                (HMENU) ID_ROTATEIMG_ANGLE,
                g_ItmagesInst,
                NULL);
        use_proxy = CreateWindowEx(
                0,
                "BUTTON",
                gettext("Use proxy server"),
                WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX | WS_TABSTOP | WS_CLIPSIBLINGS,
                CHECK_USEPROXY_X,
                CHECK_USEPROXY_Y,
                CHECK_USEPROXY_WIDTH,
                CHECK_USEPROXY_HEIGHT,
                login_wnd,
                (HMENU) ID_CHECK_USEPROXY,
                g_ItmagesInst,
                NULL);
        proxy_addrss = CreateWindowEx(
                WS_EX_CLIENTEDGE,
                "EDIT",
                "",
                WS_CHILD | WS_VISIBLE | ES_AUTOHSCROLL | WS_TABSTOP | WS_CLIPSIBLINGS,
                PROXY_ADDRESS_X,
                PROXY_ADDRESS_Y,
                PROXY_ADDRESS_WIDTH,
                PROXY_ADDRESS_HEIGHT,
                login_wnd,
                (HMENU) ID_PROXY_ADDRESS,
                g_ItmagesInst,
                NULL);
        proxy_addrss_txt = CreateWindowEx(
                0,
                "STATIC",
                gettext("Address:"),
                WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS,
                PROXYADDRESS_TXT_X,
                PROXYADDRESS_TXT_Y,
                PROXYADDRESS_TXT_WIDTH,
                PROXYADDRESS_TXT_HEIGHT,
                login_wnd,
                (HMENU) ID_PROXYADDRESS_TXT,
                g_ItmagesInst,
                NULL);
        proxy_port = CreateWindowEx(
                WS_EX_CLIENTEDGE,
                "EDIT",
                "",
                WS_CHILD | WS_VISIBLE | ES_NUMBER | ES_AUTOHSCROLL | WS_TABSTOP | WS_CLIPSIBLINGS,
                PROXY_PORT_X,
                PROXY_PORT_Y,
                PROXY_PORT_WIDTH,
                PROXY_PORT_HEIGHT,
                login_wnd,
                (HMENU) ID_PROXY_PORT,
                g_ItmagesInst,
                NULL);
        proxy_port_txt = CreateWindowEx(
                0,
                "STATIC",
                gettext("Port:"),
                WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS,
                PROXYPORT_TXT_X,
                PROXYPORT_TXT_Y,
                PROXYPORT_TXT_WIDTH,
                PROXYPORT_TXT_HEIGHT,
                login_wnd,
                (HMENU) ID_PROXYPORT_TXT,
                g_ItmagesInst,
                NULL);
        upload_progress = CreateWindowEx(
                0,
                PROGRESS_CLASS,
                NULL,
                WS_CHILD | WS_VISIBLE | PBS_SMOOTH | WS_CLIPSIBLINGS,
                UPLOAD_PROGRESSBAR_X,
                UPLOAD_PROGRESSBAR_Y,
                UPLOAD_PROGRESSBAR_WIDTH,
                UPLOAD_PROGRESSBAR_HEIGHT,
                login_wnd,
                (HMENU) ID_UPLOAD_PROGRESSBAR,
                g_ItmagesInst,
                NULL);

        about = CreateWindowEx(
                0,
                "BUTTON",
                gettext("About"),
                WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_CLIPSIBLINGS,
                ABOUT_BUTTON_X,
                ABOUT_BUTTON_Y,
                ABOUT_BUTTON_WIDTH,
                ABOUT_BUTTON_HEIGHT,
                login_wnd,
                (HMENU) ID_ABOUT_BUTTON,
                g_ItmagesInst,
                NULL);
        upload = CreateWindowEx(
                0,
                "BUTTON",
                gettext("Upload"),
                WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_CLIPSIBLINGS,
                UPLOAD_BUTTON_X,
                UPLOAD_BUTTON_Y,
                UPLOAD_BUTTON_WIDTH,
                UPLOAD_BUTTON_HEIGHT,
                login_wnd,
                (HMENU) ID_UPLOAD_BUTTON,
                g_ItmagesInst,
                NULL);

        CreateTooltip(
                login,
                login_tool,
                gettext("Enter your login here"));
        CreateTooltip(
                passwd,
                passwd_tool,
                gettext("Enter your password here"));
        CreateTooltip(
                without_login,
                withoutlogin_tool,
                gettext("Upload the image without authentification"));
        CreateTooltip(
                save_data,
                savedata_tool,
                gettext("Store the login and password in the configuration file"));
        CreateTooltip(
                resize_img,
                resizeimg_tool,
                gettext("Respectively rescales the image to the selected width"));
        CreateTooltip(
                resize_size,
                size_tool,
                gettext("Enter the new image's width here"));
        CreateTooltip(
                resize_filter,
                filter_tool,
                gettext("Select the type of the rescale filter"));
        CreateTooltip(
                proxy_addrss,
                addrss_tool,
                gettext("Enter the address of the proxy server here"));
        CreateTooltip(
                proxy_port,
                port_tool,
                gettext("Enter the port of the proxy server here"));

        // User's login and password have to be less than 32 characters.
        SendMessage(login, EM_SETLIMITTEXT, (WPARAM) 32, 0);
        SendMessage(passwd, EM_SETLIMITTEXT, (WPARAM) 32, 0);

        // Limiting width value of the image to 4 digits.
        SendMessage(resize_size, EM_SETLIMITTEXT, (WPARAM) 4, 0);

        /* Limiting possible lenght of the proxy addres and port to
        15 and 4 chacters respectively. */
        SendMessage(proxy_addrss, EM_SETLIMITTEXT, (WPARAM) 15, 0);
        SendMessage(proxy_port, EM_SETLIMITTEXT, (WPARAM) 4, 0);

        // Configuring Up-Down control.
        SendMessage(resize_updown, UDM_SETBUDDY, (LONG) resize_size, 0L);
        SendMessage(resize_updown, UDM_SETRANGE, 0, MAKELONG(9999, 0));
        SendMessage(resize_updown, UDM_SETPOS, 0, MAKELONG(0, 0));

        // Resize and rotate image checkboxes are disabled by default.
        SendMessage(resize_img, BM_SETCHECK, (WPARAM) BST_UNCHECKED, 0);
        SendMessage(rotate_img, BM_SETCHECK, (WPARAM) BST_UNCHECKED, 0);

        // Setting possible rescale filters.
        SendMessage(resize_filter, CB_ADDSTRING, 0, (LPARAM) gettext("Lanczos"));
        SendMessage(resize_filter, CB_ADDSTRING, 0, (LPARAM) gettext("Box"));
        SendMessage(resize_filter, CB_ADDSTRING, 0, (LPARAM) gettext("Bilinear"));
        SendMessage(resize_filter, CB_ADDSTRING, 0, (LPARAM) gettext("B-Spline"));

        // Setting possible rotate angles.
        SendMessage(rotate_angle, CB_ADDSTRING, 0, (LPARAM) gettext("Do not rotate"));
        SendMessage(rotate_angle, CB_ADDSTRING, 0, (LPARAM) gettext("Left"));
        SendMessage(rotate_angle, CB_ADDSTRING, 0, (LPARAM) gettext("Right"));
        SendMessage(rotate_angle, CB_ADDSTRING, 0, (LPARAM) gettext("Flip"));

        // Setting the default values for the resize filter and rotate angle.
        SendMessage(resize_filter, CB_SETCURSEL, 0, 0);
        SendMessage(rotate_angle, CB_SETCURSEL, 0, 0);

        EnableWindow(resize_size, 0);
        EnableWindow(resize_updown, 0);
        EnableWindow(resize_filter, 0);
        EnableWindow(rotate_angle, 0);

        // Seting default system font for the child windows.
        SetSystemFont(login_wnd, ID_LOGIN, ID_UPLOAD_BUTTON);

        // Hidding child windows placed on the 2nd and 3rd tab.
        HideWindows(login_wnd, ID_CHECK_RESIZEIMG, ID_PROXYPORT_TXT);

        // Seting focus on Upload button by default.
        SetFocus(upload);

        char *config_path = WideToAcp(configFiles.config);

        char state[MAX_STATE];
        memset(state, 0, MAX_STATE);

        if (GetPrivateProfileString("Settings", "savedata", NULL, state, MAX_STATE + 1, config_path) == 0) {
            memcpy(state, "false", 6);
        }

        // If the "Save account setting" checkbox was checked.
        if (memcmp(state, "true", 5) == 0) {
            userInfo.login = (char*) calloc(MAX_LOGIN + 1, sizeof (char));
            if (userInfo.login == NULL) {
                ErrorsHandler(ERROR_MEMALLOC);
                exit(EXIT_FAILURE);
            }

            userInfo.passwd = (char*) calloc(MAX_PASSWD + 1, sizeof (char));
            if (userInfo.passwd == NULL) {
                ErrorsHandler(ERROR_MEMALLOC);
                exit(EXIT_FAILURE);
            }

            // Retrieving the user's login and password from the configuration file and putting them into the Edit controls.
            if (GetPrivateProfileString("Settings", "login", NULL, userInfo.login, MAX_LOGIN + 1, config_path) == 0 ||
                    GetPrivateProfileString("Settings", "passwd", NULL, userInfo.passwd, MAX_PASSWD + 1, config_path) == 0) {
                ErrorsHandler(ERROR_READCONFFILE);
            } else {
                SetWindowText(login, userInfo.login);
                SetWindowText(passwd, userInfo.passwd);
            }
            SendMessage(save_data, BM_SETCHECK, (WPARAM) TRUE, 0);
        } else {
            EnableWindow(login, 0);
            EnableWindow(passwd, 0);
            SendMessage(without_login, BM_SETCHECK, (WPARAM) TRUE, 0);
        }

        memset(state, 0, MAX_STATE);

        if (GetPrivateProfileString("Proxy", "useproxy", NULL, state, MAX_STATE + 1, config_path) == 0) {
            memcpy(state, "false", 6);
        }

        // If "Use proxy server" checkbox was checked.
        if (memcmp(state, "true", 5) == 0) {
            userInfo.address = (char*) calloc(MAX_ADDRSS + 1, sizeof (char));
            if (userInfo.address == NULL) {
                ErrorsHandler(ERROR_MEMALLOC);
                exit(EXIT_FAILURE);
            }

            userInfo.port = (char*) calloc(MAX_PORT + 1, sizeof (char));
            if (userInfo.port == NULL) {
                ErrorsHandler(ERROR_MEMALLOC);
                exit(EXIT_FAILURE);
            }

            // Retrieving proxy address and port from the configuration file and putting them into the Edit controls.
            if (GetPrivateProfileString("Proxy", "address", NULL, userInfo.address, MAX_ADDRSS + 1, config_path) == 0 ||
                    GetPrivateProfileString("Proxy", "port", NULL, userInfo.port, MAX_PORT + 1, config_path) == 0) {
                ErrorsHandler(ERROR_READCONFFILE);
            } else {
                SetWindowText(proxy_addrss, userInfo.address);
                SetWindowText(proxy_port, userInfo.port);
            }
            SendMessage(use_proxy, BM_SETCHECK, (WPARAM) TRUE, 0);
        } else {
            EnableWindow(proxy_addrss, 0);
            EnableWindow(proxy_port, 0);
            SendMessage(use_proxy, BM_SETCHECK, (WPARAM) FALSE, 0);
        }
        free(config_path);
    }
        break;
    case WM_COMMAND:
        if (LOWORD(wParam) == ID_CHECK_WITHOUTLOGIN) {
            LRESULT result = SendMessage(without_login, BM_GETCHECK, 0, 0);
            if (result == BST_CHECKED) {
                EnableWindow(login, 0);
                EnableWindow(passwd, 0);
                SendMessage(login, EM_SETSEL, 0, -1);
                SendMessage(login, WM_CLEAR, 0, 0);
                SendMessage(passwd, EM_SETSEL, 0, -1);
                SendMessage(passwd, WM_CLEAR, 0, 0);
                SendMessage(save_data, BM_SETCHECK, (WPARAM) FALSE, 0);
            } else {
                EnableWindow(login, 1);
                EnableWindow(passwd, 1);
                SendMessage(without_login, BM_SETCHECK, (WPARAM) FALSE, 0);
            }
        }

        if (LOWORD(wParam) == ID_CHECK_SAVEDATA) {
            LRESULT result = SendMessage(without_login, BM_GETCHECK, 0, 0);
            if (result == BST_CHECKED) {
                EnableWindow(login, 1);
                EnableWindow(passwd, 1);
                SendMessage(without_login, BM_SETCHECK, (WPARAM) FALSE, 0);
            }
        }

        if (LOWORD(wParam) == ID_LOGIN && HIWORD(wParam) == EN_CHANGE) {
            if (userInfo.login == NULL) {
                userInfo.login = (char*) calloc(MAX_LOGIN + 1, sizeof (char));
                if (userInfo.login == NULL) {
                    ErrorsHandler(ERROR_MEMALLOC);
                    exit(EXIT_FAILURE);
                }
            } else {
                memset(userInfo.login, 0, MAX_LOGIN + 1);
            }
            GetWindowText(login, userInfo.login, MAX_LOGIN + 1);
        }

        if (LOWORD(wParam) == ID_PASSWD && HIWORD(wParam) == EN_CHANGE) {
            if (userInfo.passwd == NULL) {
                userInfo.passwd = (char*) calloc(MAX_PASSWD + 1, sizeof (char));
                if (userInfo.passwd == NULL) {
                    ErrorsHandler(ERROR_MEMALLOC);
                    exit(EXIT_FAILURE);
                }
            } else {
                memset(userInfo.passwd, 0, MAX_PASSWD + 1);
            }
            GetWindowText(passwd, userInfo.passwd, MAX_PASSWD + 1);
        }

        if (LOWORD(wParam) == ID_CHECK_RESIZEIMG) {
            LRESULT result = SendMessage(resize_img, BM_GETCHECK, 0, 0);
            if (result == BST_CHECKED) {
                EnableWindow(resize_size, 1);
                EnableWindow(resize_updown, 1);
                EnableWindow(resize_filter, 1);
            } else {
                EnableWindow(resize_size, 0);
                EnableWindow(resize_updown, 0);
                EnableWindow(resize_filter, 0);
            }
        }

        if (LOWORD(wParam) == ID_CHECK_ROTATEIMG) {
            LRESULT result = SendMessage(rotate_img, BM_GETCHECK, 0, 0);
            if (result == BST_CHECKED) {
                EnableWindow(rotate_angle, 1);
            } else {
                EnableWindow(rotate_angle, 0);
            }
        }

        if (LOWORD(wParam) == ID_RESIZEIMG_SIZE && HIWORD(wParam) == EN_CHANGE) {
            if (imageInfo.width == 0) {
                ErrorsHandler(ERROR_GETIMGWIDTH);
            } else {
                char width[MAX_DIM_CHARS];
                memset(width, 0, MAX_DIM_CHARS);

                GetWindowText(resize_size, width, MAX_DIM_CHARS + 1);
                size_t new_width = atoi(width);

                // Reduce value have to be less than actual width of the image.
                if (new_width > imageInfo.width) {
                    ErrorsHandler(ERROR_BIGWIDTH);
                    SendMessage(resize_size, EM_SETSEL, 0, -1);
                    SendMessage(resize_size, WM_CLEAR, 0, 0);
                }
            }
        }

        if (LOWORD(wParam) == ID_CHECK_USEPROXY) {
            LRESULT result = SendMessage(use_proxy, BM_GETCHECK, 0, 0);
            if (result == BST_CHECKED) {
                EnableWindow(proxy_addrss, 1);
                EnableWindow(proxy_port, 1);
            } else {
                EnableWindow(proxy_addrss, 0);
                EnableWindow(proxy_port, 0);
                SendMessage(proxy_addrss, EM_SETSEL, 0, -1);
                SendMessage(proxy_addrss, WM_CLEAR, 0, 0);
                SendMessage(proxy_port, EM_SETSEL, 0, -1);
                SendMessage(proxy_port, WM_CLEAR, 0, 0);
            }
        }

        if (LOWORD(wParam) == ID_PROXY_ADDRESS && HIWORD(wParam) == EN_CHANGE) {
            GetWindowText(proxy_addrss, userInfo.address, 16);
        }

        if (LOWORD(wParam) == ID_PROXY_PORT && HIWORD(wParam) == EN_CHANGE) {
            GetWindowText(proxy_port, userInfo.port, 5);
        }

        // If the "About" button was pressed.
        if (LOWORD(wParam) == ID_ABOUT_BUTTON) {
            CreateAboutWnd();
            EnableWindow(login_wnd, 0);
        }

        // If the "Upload" button was pressed.
        if (LOWORD(wParam) == ID_UPLOAD_BUTTON) {
            EnableWindow(upload, 0);

            char *config_path = WideToAcp(configFiles.config);

            LRESULT resize_result = SendMessage(resize_img, BM_GETCHECK, 0, 0);
            LRESULT rotate_result = SendMessage(rotate_img, BM_GETCHECK, 0, 0);

            if (resize_result == BST_CHECKED || rotate_result == BST_CHECKED) {
                size_t choise = MessageBox(
                        login_wnd,
                        gettext("The current image was modified.\nApply the changes?"),
                        gettext("ITmages - Image"),
                        MB_YESNO | MB_ICONQUESTION);

                if (choise == IDYES) {
                    if (resize_result == BST_CHECKED) {
                        char width[MAX_DIM_CHARS];
                        memset(width, 0, MAX_DIM_CHARS);

                        GetWindowText(resize_size, width, MAX_DIM_CHARS + 1);

                        size_t new_width = atoi(width);
                        size_t filter_index = SendMessage(resize_filter, CB_GETCURSEL, 0, 0);

                        ResizeImageTo(imageData.path, new_width, filter_index);
                        imageInfo.size = GetImageSize(imageData.path);
                    }

                    if (rotate_result == BST_CHECKED) {
                        int angle;

                        size_t selected_index = SendMessage(rotate_angle, CB_GETCURSEL, 0, 0);
                        switch (selected_index) {
                        case 0: angle = 0; // Do not rotate
                            break;
                        case 1: angle = 90; // Left
                            break;
                        case 2: angle = -90; // Rigth
                            break;
                        case 3: angle = 180; // Flip
                            break;
                        }
                        RotateImageTo(imageData.path, angle);
                        imageInfo.size = GetImageSize(imageData.path);
                    }

					/* Retrieving the information about the image again, coz the
                    image's properties was changed. */
                    FreeImageInfo(&imageInfo);
                    if (GetImageInfo(imageData.path, imageData.basename,
                            imageData.ext, &imageInfo) == -1) {
                        ErrorsHandler(ERROR_GETIMGINFO);
                        exit(EXIT_FAILURE);
                    }
                }
            }

            /* If the "Save account settings" checkbox was checked - saving login
            and password to the configuration file. */
            LRESULT result_savedata = SendMessage(save_data, BM_GETCHECK, 0, 0);
            if (result_savedata == BST_CHECKED) {
                if (WritePrivateProfileString("Settings", "login", userInfo.login, config_path) == 0 ||
                        WritePrivateProfileString("Settings", "passwd", userInfo.passwd, config_path) == 0 ||
                        WritePrivateProfileString("Settings", "savedata", "true", config_path) == 0) {
                    ErrorsHandler(ERROR_WRITECONFFILE);
                }
            } else {
                if (WritePrivateProfileString("Settings", "savedata", "false", config_path) == 0) {
                    ErrorsHandler(ERROR_WRITECONFFILE);
                }
            }

            /* If the "Use proxy server" checkbox was checked - saving proxy
            address and port to the configuration file. */
            LRESULT result_useproxy = SendMessage(use_proxy, BM_GETCHECK, 0, 0);
            if (result_useproxy == BST_CHECKED) {
                if (WritePrivateProfileString("Proxy", "address", userInfo.address, config_path) == 0 ||
                        WritePrivateProfileString("Proxy", "port", userInfo.port, config_path) == 0 ||
                        WritePrivateProfileString("Proxy", "useproxy", "true", config_path) == 0) {
                    ErrorsHandler(ERROR_WRITECONFFILE);
                }
            } else {
                if (WritePrivateProfileString("Proxy", "useproxy", "false", config_path) == 0) {
                    ErrorsHandler(ERROR_WRITECONFFILE);
                }
            }

            /* Checking the image's size again to be sure that the image still
            has an appropriate size. */
            if (imageInfo.size == -1) {
                ErrorsHandler(ERROR_MAXIMGSIZE);
                exit(EXIT_FAILURE);
            } else if (imageInfo.size == 0) {
                ErrorsHandler(ERROR_BADIMGSIZE);
                exit(EXIT_FAILURE);
            } else {
                // Starting the upload threads.
                LRESULT result_withoutlogin = SendMessage(without_login, BM_GETCHECK, 0, 0);
                if (result_withoutlogin == BST_CHECKED) {
                    HANDLE withoutlogin_thread = (HANDLE) _beginthreadex(NULL, 0, (void*) UploadWithoutLoginThread, login_wnd, 0, NULL);
                    if (withoutlogin_thread == NULL) {
                        ErrorsHandler(ERROR_CREATETHREAD);
                    }
                    CloseHandle(withoutlogin_thread);
                } else {
                    HANDLE withlogin_thread = (HANDLE) _beginthreadex(NULL, 0, (void*) UploadWithLoginThread, login_wnd, 0, NULL);
                    if (withlogin_thread == NULL) {
                        ErrorsHandler(ERROR_CREATETHREAD);
                    }
                    CloseHandle(withlogin_thread);
                }
            }
            free(config_path);
        }
        break;
    case WM_NOTIFY:
    {
        LPNMHDR lpnmhdr = (LPNMHDR) lParam;
        if (lpnmhdr->code == TCN_SELCHANGE) {
            size_t nTab = TabCtrl_GetCurSel(login_tab);
            switch (nTab) {
            case 0:
                ShowTabWindows(
                        login_wnd,
                        ID_LOGIN,
                        ID_PROXYPORT_TXT,
                        ID_LOGIN,
                        ID_CHECK_SAVEDATA);
                break;
            case 1:
                ShowTabWindows(
                        login_wnd,
                        ID_LOGIN,
                        ID_PROXYPORT_TXT,
                        ID_CHECK_RESIZEIMG,
                        ID_ROTATEIMG_ANGLE);
                break;
            case 2:
                ShowTabWindows(
                        login_wnd,
                        ID_LOGIN,
                        ID_PROXYPORT_TXT,
                        ID_CHECK_USEPROXY,
                        ID_PROXYPORT_TXT);
                break;
            }
        }
    }
        break;
    case WM_FINISHTHREAD:
        /* If image upload was successful - creating the Ready window and minimizing
        the Login window into the taskbar. */
        CreateReadyWnd();
        EnableWindow(login_wnd, 0);
        break;
    case WM_SYSCOLORCHANGE:
        /* Updating the statics' and checkboxes' background if the system's
        visual style was changed. */
        UpdateBackgroundBrush(login_tab);
        break;
    case WM_CTLCOLORSTATIC:
        /* If the visual style is enabled - drawing an appropriate background
        for the static controls and checkboxes. */
        if (g_ThemeActive == 1) {
            SetThemedBackground(login_tab, (HWND) lParam, (HDC) wParam);
            return (LRESULT) (g_StaticBrush);
        } else {
            SetBkColor((HDC) wParam, (COLORREF) GetSysColor(COLOR_BTNFACE));
            return (LRESULT) GetSysColorBrush(COLOR_BTNFACE);
        }
        break;
    case WM_DESTROY:
        FreeImageInfo(&imageInfo);
        FreeImageLinks(&imageLinks);
        FreeImageData(&imageData);
        FreeUserInfo(&userInfo);
        FreeConfig(&configFiles);

        PostQuitMessage(0);
        break;
    default:
        return DefWindowProc(login_wnd, message, wParam, lParam);
    }
    return 0;
}

INT APIENTRY WinAbout()
{
    WNDCLASSEX itmagesAbout;
    memset(&itmagesAbout, 0, sizeof (itmagesAbout));

    itmagesAbout.cbSize = sizeof (itmagesAbout);
    itmagesAbout.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
    itmagesAbout.lpfnWndProc = WinAboutProc;
    itmagesAbout.cbClsExtra = 0;
    itmagesAbout.cbWndExtra = 0;
    itmagesAbout.hInstance = g_ItmagesInst;
    itmagesAbout.hIcon = g_AppIcon;
    itmagesAbout.hCursor = LoadCursor(NULL, IDC_ARROW);
    itmagesAbout.hbrBackground = (HBRUSH) (COLOR_BTNFACE + 1);
    itmagesAbout.lpszMenuName = NULL;
    itmagesAbout.lpszClassName = g_AboutWindowClass;
    itmagesAbout.hIconSm = NULL;

    if (RegisterClassEx(&itmagesAbout) == 0) {
        return -1;
    }
    return 0;
}

void CreateAboutWnd()
{
    size_t frame_width = GetSystemMetrics(SM_CXSIZEFRAME);
    size_t frame_height = GetSystemMetrics(SM_CYSIZEFRAME);

    RECT parent_rect;
    GetWindowRect(login_wnd, &parent_rect);

    about_wnd = CreateWindowEx(
            WS_EX_DLGMODALFRAME,
            g_AboutWindowClass,
            gettext("ITmages - About"),
            WS_POPUP | WS_VISIBLE | WS_SYSMENU | WS_CAPTION | WS_THICKFRAME | WS_CLIPCHILDREN,
            (parent_rect.left + LOGIN_WND_WIDTH / 2) - ABOUT_WND_WIDTH / 2,
            (parent_rect.top + LOGIN_WND_HEIGHT / 2) - ABOUT_WND_HEIGHT / 2,
            ABOUT_WND_WIDTH + frame_width,
            ABOUT_WND_HEIGHT + frame_height,
            login_wnd,
            NULL,
            g_ItmagesInst,
            NULL);

    // Creating the Tab conrol for the About window
    CreateAboutTabControl();

    ShowWindow(about_wnd, SW_SHOW);
    UpdateWindow(about_wnd);
}

void CreateAboutTabControl()
{
    HWND about_tab = CreateWindowEx(
            0,
            WC_TABCONTROL,
            "",
            WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_CLIPCHILDREN,
            ABOUT_TABCONTROL_X,
            ABOUT_TABCONTROL_Y,
            ABOUT_TABCONTROL_WIDTH,
            ABOUT_TABCONTROL_HEIGHT,
            about_wnd,
            (HMENU) ID_ABOUT_TABCONTROL,
            g_ItmagesInst,
            NULL);

    SendMessage(about_tab, WM_SETFONT, (WPARAM) g_SystemFont, 0);

    TCITEM tie;
    memset(&tie, 0, sizeof (tie));

    tie.mask = TCIF_TEXT;
    tie.pszText = gettext("About");
    TabCtrl_InsertItem(about_tab, 0, &tie);
    tie.pszText = gettext("License");
    TabCtrl_InsertItem(about_tab, 1, &tie);
    tie.pszText = gettext("Credits");
    TabCtrl_InsertItem(about_tab, 2, &tie);

    UpdateBackgroundBrush(about_tab);
}

LRESULT CALLBACK WinAboutProc(
        HWND about_wnd,
        UINT message,
        WPARAM wParam,
        LPARAM lParam)
{
    HWND about_tab = GetDlgItem(about_wnd, ID_ABOUT_TABCONTROL);
    static HWND itmages_img,
            about_info,
            license_info,
            credits_info,
            close;

    size_t frame_width = GetSystemMetrics(SM_CXSIZEFRAME);
    size_t frame_height = GetSystemMetrics(SM_CYSIZEFRAME);

    switch (message) {
    case WM_CREATE:
        itmages_img = CreateWindowEx(
                0,
                "STATIC",
                "",
                WS_CHILD | WS_VISIBLE | SS_ICON | WS_CLIPSIBLINGS,
                ITMAGES_IMG_X,
                ITMAGES_IMG_Y,
                CW_USEDEFAULT,
                CW_USEDEFAULT,
                about_wnd,
                (HMENU) ID_ITMAGES_IMG,
                g_ItmagesInst,
                NULL);
        about_info = CreateWindowEx(
                0,
                "STATIC",
                gettext("ITmages upload client\n"
                "Version : 0.3\n"
                "Author : Dmitriy Simbiriatin\n"
                "E - mail : slpiv@itmages.ru\n"
                "Copyright(c) 2011 Dmitriy Simbiriatin\n"
                "License : GPL version 2"),
                WS_CHILD | WS_VISIBLE | ES_LEFT | WS_CLIPSIBLINGS,
                ABOUT_INFO_X,
                ABOUT_INFO_Y,
                ABOUT_INFO_WIDTH,
                ABOUT_INFO_HEIGHT,
                about_wnd,
                (HMENU) ID_ABOUT_INFO,
                g_ItmagesInst,
                NULL);
        license_info = CreateWindowEx(
                0,
                "EDIT",
                "",
                WS_CHILD | WS_VISIBLE | ES_LEFT | ES_MULTILINE |
                WS_VSCROLL | ES_READONLY | ES_WANTRETURN |
                ES_AUTOVSCROLL | WS_CLIPSIBLINGS,
                LICENSE_INFO_X,
                LICENSE_INFO_Y,
                LICENSE_INFO_WIDTH,
                LICENSE_INFO_HEIGHT,
                about_wnd,
                (HMENU) ID_LICENSE_INFO,
                g_ItmagesInst,
                NULL);
        credits_info = CreateWindowEx(
                0,
                "EDIT",
                "",
                WS_CHILD | WS_VISIBLE | ES_LEFT | ES_MULTILINE |
                WS_VSCROLL | ES_READONLY | ES_WANTRETURN |
                ES_AUTOVSCROLL | WS_CLIPSIBLINGS,
                CREDITS_INFO_X,
                CREDITS_INFO_Y,
                CREDITS_INFO_WIDTH,
                CREDITS_INFO_HEIGHT,
                about_wnd,
                (HMENU) ID_CREDITS_INFO,
                g_ItmagesInst,
                NULL);
        close = CreateWindowEx(
                0,
                "BUTTON",
                gettext("Close"),
                WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_CLIPSIBLINGS,
                CLOSE_BUTTON_X,
                CLOSE_BUTTON_Y,
                CLOSE_BUTTON_WIDTH,
                CLOSE_BUTTON_HEIGHT,
                about_wnd,
                (HMENU) ID_CLOSE_BUTTON,
                g_ItmagesInst,
                NULL);

        // Putting ITmages logo into the static control.
        SendMessage(itmages_img, STM_SETIMAGE, IMAGE_ICON, (LPARAM) g_AppIcon);

        // Hiding the windows that are placed on the 2nd and the 3rd tabs.
        HideWindows(about_wnd, ID_LICENSE_INFO, ID_CREDITS_INFO);

        // Setting the system's default font for the child windows.
        SetSystemFont(about_wnd, ID_ABOUT_INFO, ID_CLOSE_BUTTON);

        // Loading the documentation files.
        if (LoadDocFile(license_info, DOCS_DIR, "COPYING.txt") == -1) {
			SetWindowText(license_info, gettext("Failed to open the file."));
		}
        if (LoadDocFile(credits_info, DOCS_DIR, "CREDITS.txt") == -1) {
			SetWindowText(credits_info, gettext("Failed to open the file."));
		}
        break;
    case WM_COMMAND:
        if (LOWORD(wParam) == ID_CLOSE_BUTTON) {
            DestroyWindow(about_wnd);
        }
        break;
    case WM_NOTIFY:
    {
        LPNMHDR lpnmhdr = (LPNMHDR) lParam;
        if (lpnmhdr->code == TCN_SELCHANGE) {
            size_t nTab = TabCtrl_GetCurSel(about_tab);
            switch (nTab) {
            case 0:
                ShowTabWindows(
                        about_wnd,
                        ID_ITMAGES_IMG,
                        ID_CREDITS_INFO,
                        ID_ITMAGES_IMG,
                        ID_ABOUT_INFO);
                break;
            case 1:
                ShowTabWindows(
                        about_wnd,
                        ID_ITMAGES_IMG,
                        ID_CREDITS_INFO,
                        ID_LICENSE_INFO,
                        ID_LICENSE_INFO);
                break;
            case 2:
                ShowTabWindows(
                        about_wnd,
                        ID_ITMAGES_IMG,
                        ID_CREDITS_INFO,
                        ID_CREDITS_INFO,
                        ID_CREDITS_INFO);
                break;
            }
        }
    }
        break;
    case WM_SYSCOLORCHANGE:
        /* Updating the statics' and checkboxes' background if the system's
        visual style was changed. */
        UpdateBackgroundBrush(about_tab);
        break;
    case WM_CTLCOLORSTATIC:
        /* If the visual style is enabled - drawing an appropriate background
        for the static controls and checkboxes. */
        if (g_ThemeActive == 1) {
            SetThemedBackground(about_tab, (HWND) lParam, (HDC) wParam);
            return (LRESULT) g_StaticBrush;
        } else {
            SetBkColor((HDC) wParam, (COLORREF) GetSysColor(COLOR_BTNFACE));
            return (LRESULT) GetSysColorBrush(COLOR_BTNFACE);
        }
        break;
    case WM_SIZING:
    {
        UpdateBackgroundBrush(about_tab);

        RECT about_rect;
        GetWindowRect(about_wnd, &about_rect);

        // Difference between the initial size of the window and the new one.
        int delta_width = ((about_rect.right - about_rect.left) - ABOUT_WND_WIDTH);
        int delta_height = ((about_rect.bottom - about_rect.top) - ABOUT_WND_HEIGHT);

        MoveWindow(about_tab,
                ABOUT_TABCONTROL_X,
                ABOUT_TABCONTROL_Y,
                ABOUT_TABCONTROL_WIDTH + delta_width - frame_width,
                ABOUT_TABCONTROL_HEIGHT + delta_height - frame_height,
                TRUE);
        MoveWindow(about_info,
                ABOUT_INFO_X,
                ABOUT_INFO_Y,
                ABOUT_INFO_WIDTH + delta_width - frame_width,
                ABOUT_INFO_HEIGHT + delta_height - frame_height,
                TRUE);
        MoveWindow(license_info,
                LICENSE_INFO_X,
                LICENSE_INFO_Y,
                LICENSE_INFO_WIDTH + delta_width - frame_width,
                LICENSE_INFO_HEIGHT + delta_height - frame_height,
                TRUE);
        MoveWindow(credits_info,
                CREDITS_INFO_X,
                CREDITS_INFO_Y,
                CREDITS_INFO_WIDTH + delta_width - frame_width,
                CREDITS_INFO_HEIGHT + delta_height - frame_height,
                TRUE);
        MoveWindow(close,
                CLOSE_BUTTON_X + delta_width - frame_width,
                CLOSE_BUTTON_Y + delta_height - frame_height,
                CLOSE_BUTTON_WIDTH,
                CLOSE_BUTTON_HEIGHT,
                TRUE);
    }
        break;
    case WM_GETMINMAXINFO:
    {
        // Setting the minimal width and height of the About window.
        ((MINMAXINFO*) lParam)->ptMinTrackSize.x = ABOUT_WND_WIDTH + frame_width;
        ((MINMAXINFO*) lParam)->ptMinTrackSize.y = ABOUT_WND_HEIGHT + frame_height;
    }
        break;
    case WM_DESTROY:
        EnableWindow(login_wnd, 1);
        SetFocus(login_wnd);
        DestroyWindow(about_wnd);
        break;
    default:
        return DefWindowProc(about_wnd, message, wParam, lParam);
    }
    return 0;
}

INT APIENTRY WinReady()
{
    WNDCLASSEX itmagesReady;
    memset(&itmagesReady, 0, sizeof (itmagesReady));

    itmagesReady.cbSize = sizeof (itmagesReady);
    itmagesReady.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
    itmagesReady.lpfnWndProc = WinReadyProc;
    itmagesReady.cbClsExtra = 0;
    itmagesReady.cbWndExtra = 0;
    itmagesReady.hInstance = g_ItmagesInst;
    itmagesReady.hIcon = g_AppIcon;
    itmagesReady.hCursor = LoadCursor(NULL, IDC_ARROW);
    itmagesReady.hbrBackground = (HBRUSH) (COLOR_BTNFACE + 1);
    itmagesReady.lpszMenuName = NULL;
    itmagesReady.lpszClassName = g_ReadyWindowClass;
    itmagesReady.hIconSm = NULL;

    if (RegisterClassEx(&itmagesReady) == 0) {
        return -1;
    }
    return 0;
}

void CreateReadyWnd()
{
    RECT parent_rect;
    GetWindowRect(login_wnd, &parent_rect);

    ready_wnd = CreateWindowEx(
            WS_EX_DLGMODALFRAME,
            g_ReadyWindowClass,
            gettext("ITmages - Ready!"),
            WS_POPUP | WS_VISIBLE | WS_SYSMENU | WS_CAPTION | WS_CLIPCHILDREN,
            (parent_rect.left + LOGIN_WND_WIDTH / 2) - READY_WND_WIDTH / 2,
            (parent_rect.top + LOGIN_WND_HEIGHT / 2) - READY_WND_HEIGHT / 2,
            READY_WND_WIDTH,
            READY_WND_HEIGHT,
            login_wnd,
            NULL,
            g_ItmagesInst,
            NULL);

    // Creating a preview for the uploaded image.
    HWND preview = GetDlgItem(ready_wnd, ID_IMAGE_PREVIEW);
    previewImage = (HBITMAP) HBITMAPFromFile(
            preview,
            imageData.path,
            IMAGE_PREVIEW_WIDTH,
            IMAGE_PREVIEW_HEIGHT);

    if (previewImage != NULL) {
        SendMessage(preview, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) previewImage);
    }

    ShowWindow(ready_wnd, SW_SHOW);
    UpdateWindow(ready_wnd);
}

LRESULT CALLBACK WinReadyProc(
        HWND ready_wnd,
        UINT message,
        WPARAM wParam,
        LPARAM lParam)
{
    static HWND preview,
            links_list,
            link_preview,
            copy_to_clip,
            open_in_bsr;

    static HWND links_list_tool,
            preview_tool,
            clip_tool,
            bsr_tool;

    static char image_info[MAX_INFO];
    static char preview_buffer[MAX_PREVIEW];

    switch (message) {
    case WM_PAINT:
    {
        DefWindowProc(ready_wnd, message, wParam, lParam);

        HDC ready_wnd_dc = GetDC(ready_wnd);

        RECT text_rect;
        RECT edge_rect;

        SetRect(&text_rect, 18, 8, 165, 25);
        SetRect(&edge_rect, 10, 15, 170, 140);

        SetBkColor(ready_wnd_dc, GetSysColor(COLOR_BTNFACE));
        SelectObject(ready_wnd_dc, g_SystemFont);

        // Drawing the groupbox around the preview image.
        DrawEdge(ready_wnd_dc, &edge_rect, BDR_SUNKENINNER | BDR_RAISEDOUTER, BF_RECT | BF_FLAT);
        DrawText(ready_wnd_dc, gettext("Preview"), -1, &text_rect, DT_LEFT | DT_VCENTER);

        DeleteDC(ready_wnd_dc);
        ReleaseDC(ready_wnd, ready_wnd_dc);
    }
        break;
    case WM_CREATE:
        preview = CreateWindowEx(
                0,
                "STATIC",
                "",
                WS_CHILD | WS_VISIBLE | SS_BITMAP | SS_NOTIFY | WS_BORDER | WS_CLIPSIBLINGS,
                IMAGE_PREVIEW_X,
                IMAGE_PREVIEW_Y,
                IMAGE_PREVIEW_WIDTH,
                IMAGE_PREVIEW_HEIGHT,
                ready_wnd,
                (HMENU) ID_IMAGE_PREVIEW,
                g_ItmagesInst,
                NULL);
        links_list = CreateWindowEx(
                0,
                "COMBOBOX",
                NULL,
                WS_CHILD | WS_VISIBLE | WS_VSCROLL | CBS_DROPDOWNLIST | CBS_HASSTRINGS | WS_TABSTOP | WS_CLIPSIBLINGS,
                LINK_LIST_X,
                LINK_LIST_Y,
                LINK_LIST_WIDTH,
                LINK_LIST_HEIGHT,
                ready_wnd,
                (HMENU) ID_LINKS_LIST,
                g_ItmagesInst,
                NULL);
        link_preview = CreateWindowEx(
                WS_EX_CLIENTEDGE,
                "EDIT",
                NULL,
                WS_CHILD | WS_VISIBLE | ES_READONLY | ES_AUTOHSCROLL | WS_TABSTOP | WS_CLIPSIBLINGS,
                LINK_PREVIEW_X,
                LINK_PREVIEW_Y,
                LINK_PREVIEW_WIDTH,
                LINK_PREVIEW_HEIGHT,
                ready_wnd,
                (HMENU) ID_LINK_PREVIEW,
                g_ItmagesInst,
                NULL);
        copy_to_clip = CreateWindowEx(
                0,
                "BUTTON",
                gettext("Copy to Clipboard"),
                WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_CLIPSIBLINGS,
                COPYTOCLIPBOARD_X,
                COPYTOCLIPBOARD_Y,
                COPYTOCLIPBOARD_WIDTH,
                COPYTOCLIPBOARD_HEIGHT,
                ready_wnd,
                (HMENU) ID_COPYTOCLIPBOARD,
                g_ItmagesInst,
                NULL);
        open_in_bsr = CreateWindowEx(
                0,
                "BUTTON",
                gettext("Open in Browser"),
                WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_CLIPSIBLINGS,
                OPENINBROWSER_X,
                OPENINBROWSER_Y,
                OPENINBROWSER_WIDTH,
                OPENINBROWSER_HEIGHT,
                ready_wnd,
                (HMENU) ID_OPENINBROWSER,
                g_ItmagesInst,
                NULL);

        CreateTooltip(
                links_list,
                links_list_tool,
                gettext("Choose the link type"));
        CreateTooltip(
                copy_to_clip,
                clip_tool,
                gettext("Copy the selected link into clipboard"));
        CreateTooltip(
                open_in_bsr,
                bsr_tool,
                gettext("Open the selected link in the default web browser"));

        // Putting information about the image inside the preview's tooltip.
        wchar_t *fullname_wide = Utf8ToWide(imageInfo.fullname);
        char *fullname_mb = WideToAcp(fullname_wide);
        free(fullname_wide);

        sprintf(
                image_info,
                gettext("Name: %s\nWidth: %d\nHeight: %d\nSize: %d Kb"),
                fullname_mb,
                imageInfo.width,
                imageInfo.height,
                (imageInfo.size / 1024));
        free(fullname_mb);

        CreateTooltip(
                preview,
                preview_tool,
                image_info);

        // Setting the system's default font for the child windows.
        SetSystemFont(ready_wnd, ID_IMAGE_PREVIEW, ID_OPENINBROWSER);

        // Adding the supported link types to the list.
        SendMessage(links_list, CB_ADDSTRING, 0, (LPARAM) gettext("Link"));
        SendMessage(links_list, CB_ADDSTRING, 0, (LPARAM) gettext("Direct Link"));
        SendMessage(links_list, CB_ADDSTRING, 0, (LPARAM) gettext("BB code thumbnail"));
        SendMessage(links_list, CB_ADDSTRING, 0, (LPARAM) gettext("HTML Code thumbnail"));

        char *config_path = WideToAcp(configFiles.config);

        char index_buffer[MAX_INDEX];
        memset(index_buffer, 0, MAX_INDEX);

        // Retrieving an index of the previously chosen link type from the configuration file.
        if (GetPrivateProfileString("Settings", "linkindex", NULL, index_buffer, MAX_INDEX + 1, config_path) == 0) {
            SendMessage(links_list, CB_SETCURSEL, 0, 0);
            ShowSelectedLink(link_preview, open_in_bsr, preview_buffer, 0);
        } else {
            size_t selected_index = atoi(index_buffer);
            SendMessage(links_list, CB_SETCURSEL, selected_index, 0);
            ShowSelectedLink(link_preview, open_in_bsr, preview_buffer, selected_index);
        }
        free(config_path);
        break;
    case WM_COMMAND:
        if (LOWORD(wParam) == ID_LINKS_LIST && HIWORD(wParam) == CBN_SELCHANGE) {
            size_t selected_index = SendMessage(links_list, CB_GETCURSEL, 0, 0);
            ShowSelectedLink(link_preview, open_in_bsr, preview_buffer, selected_index);
        }
        if (LOWORD(wParam) == ID_COPYTOCLIPBOARD) {
            if (OpenClipboard(ready_wnd) == 0) {
                ErrorsHandler(ERROR_OPENCLIPBOARD);
            } else {
                HGLOBAL clip_buffer;
                char *tmp_buffer;

                EmptyClipboard();
                clip_buffer = GlobalAlloc(GMEM_DDESHARE, sizeof (preview_buffer));
                tmp_buffer = (char*) GlobalLock(clip_buffer);
                memcpy(tmp_buffer, preview_buffer, strlen(preview_buffer) + 1);
                GlobalUnlock(clip_buffer);
                SetClipboardData(CF_TEXT, clip_buffer);
                CloseClipboard();
            }
        }
        if (LOWORD(wParam) == ID_OPENINBROWSER) {
            size_t selected_index = SendMessage(links_list, CB_GETCURSEL, 0, 0);
            if (selected_index == 0 || selected_index == 1) {
                ShellExecute(NULL, "open", preview_buffer, NULL, NULL, SW_SHOWNORMAL);
            }
        }
        break;
    case WM_DESTROY:
    {
        char index_buffer[MAX_INDEX];
        memset(index_buffer, 0, MAX_INDEX);

        size_t selected_index = SendMessage(links_list, CB_GETCURSEL, 0, 0);
        itoa(selected_index, index_buffer, 10);

        char *config_path = WideToAcp(configFiles.config);

        // Saving the last chosen link type index to the configuration file.
        if (WritePrivateProfileString("Settings", "linkindex", index_buffer, config_path) == 0) {
            ErrorsHandler(ERROR_WRITECONFFILE);
        }
        free(config_path);

        DeleteObject(previewImage);

        FreeImageInfo(&imageInfo);
        FreeImageLinks(&imageLinks);
        FreeImageData(&imageData);
        FreeUserInfo(&userInfo);
        FreeConfig(&configFiles);

        EnableWindow(login_wnd, 1);
        SetFocus(login_wnd);
        
        PostQuitMessage(0);
    }
        break;
    default:
        return DefWindowProc(ready_wnd, message, wParam, lParam);
    }
    return 0;
}

UINT APIENTRY UploadWithLoginThread(HWND login_wnd)
{
    short thread_success = 1;
    short result = 0;

    char *cookies_path = WideToAcp(configFiles.cookies);

    result = ItmagesCheckConnection(cookies_path, userInfo.address, userInfo.port);
    if (result == 1) {
        ErrorsHandler(ERROR_CONNECTION);
        thread_success = 0;
    } else if (result == -1) {
        thread_success = 0;
    } else {
        result = ItmagesLogin(userInfo.login, userInfo.passwd, cookies_path,
                userInfo.address, userInfo.port);
        if (result == 1) {
            ErrorsHandler(ERROR_LOGIN);
            thread_success = 0;
        } else if (result == -1) {
            thread_success = 0;
        } else {
            FreeImageLinks(&imageLinks);
            HWND upload_progress = GetDlgItem(login_wnd, ID_UPLOAD_PROGRESSBAR);
            result = ItmagesUpload(upload_progress, imageData.path, imageInfo.fullname, imageInfo.mime,
                    cookies_path, userInfo.address, userInfo.port, imageInfo.size, &imageLinks);
            if (result != 0) {
                thread_success = 0;
            }
            result = ItmagesLogout(cookies_path, userInfo.address, userInfo.port);
            if (result == 1) {
                ErrorsHandler(ERROR_LOGOUT);
            }
        }
    }
    free(cookies_path);

    if (thread_success == 1) {
        SendMessage(login_wnd, WM_FINISHTHREAD, 0, 0);
    }
    HWND upload = GetDlgItem(login_wnd, ID_UPLOAD_BUTTON);
    EnableWindow(upload, 1);
    return (DWORD) 0;
}

UINT APIENTRY UploadWithoutLoginThread(HWND login_wnd)
{
    short thread_success = 1;
    short result = 0;

    char *cookies_path = WideToAcp(configFiles.cookies);

    result = ItmagesCheckConnection(cookies_path, userInfo.address, userInfo.port);
    if (result == 1) {
        ErrorsHandler(ERROR_CONNECTION);
        thread_success = 0;
    } else if (result == -1) {
        thread_success = 0;
    } else {
        FreeImageLinks(&imageLinks);
        HWND upload_progress = GetDlgItem(login_wnd, ID_UPLOAD_PROGRESSBAR);
        result = ItmagesUpload(upload_progress, imageData.path, imageInfo.fullname, imageInfo.mime,
                cookies_path, userInfo.address, userInfo.port, imageInfo.size, &imageLinks);
        if (result != 0) {
            thread_success = 0;
        }
    }
    free(cookies_path);

    if (thread_success == 1) {
        SendMessage(login_wnd, WM_FINISHTHREAD, 0, 0);
    }
    HWND upload = GetDlgItem(login_wnd, ID_UPLOAD_BUTTON);
    EnableWindow(upload, 1);
    return (DWORD) 0;
}

void CreateTooltip(
        HWND parent_wnd,
        HWND tooltip_wnd,
        char *tooltip_txt)
{
    tooltip_wnd = CreateWindowEx(
            WS_EX_TOPMOST,
            TOOLTIPS_CLASS,
            NULL,
            TTS_NOPREFIX | TTS_ALWAYSTIP,
            CW_USEDEFAULT,
            CW_USEDEFAULT,
            CW_USEDEFAULT,
            CW_USEDEFAULT,
            parent_wnd,
            NULL,
            g_ItmagesInst,
            NULL);

    SetWindowPos(
            tooltip_wnd,
            HWND_TOPMOST,
            0, 0, 0, 0,
            SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);

    TOOLINFO ti;
    memset(&ti, 0, sizeof (ti));

    ti.cbSize = sizeof (TOOLINFO);
    ti.uFlags = TTF_IDISHWND | TTF_SUBCLASS | TTF_TRANSPARENT;
    ti.hwnd = parent_wnd;
    ti.uId = (UINT_PTR) parent_wnd;
    ti.hinst = g_ItmagesInst;
    ti.lpszText = tooltip_txt;

    GetClientRect(parent_wnd, &ti.rect);

    SendMessage(tooltip_wnd, TTM_ADDTOOL, 0, (LPARAM) (LPTOOLINFO) & ti);
    SendMessage(tooltip_wnd, TTM_ACTIVATE, TRUE, 0);
    SendMessage(tooltip_wnd, TTM_SETMAXTIPWIDTH, 0, (WPARAM) 180);
}

char *CreateOpenImageDialog()
{
    OPENFILENAME ofn;
    memset(&ofn, 0, sizeof (OPENFILENAME));

    char *image_path = (char*) calloc(_MAX_PATH + 1, sizeof (char));
    if (image_path == NULL) {
        ErrorsHandler(ERROR_MEMALLOC);
        exit(EXIT_FAILURE);
    }

    ofn.lStructSize = sizeof (OPENFILENAME);
    ofn.lpstrFile = image_path;
    ofn.nMaxFile = _MAX_PATH;
    ofn.lpstrFilter = "*.png,*.jpeg,*.jpg,*.gif\0*.jpg;*.jpeg;*.png;*.gif\0";
    ofn.nFilterIndex = 1;
    ofn.lpstrTitle = gettext("Choose the image...");
    ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;
    if (GetOpenFileName(&ofn) == FALSE) {
        exit(EXIT_SUCCESS);
    }
    return image_path;
}

void ShowSelectedLink(
        HWND link_preview,
        HWND bsr_button,
        char *preview_buffer,
        size_t index)
{
    switch (index) {
    case 0:
        sprintf(
                preview_buffer,
                "http://itmages.ru/image/view/%s/%s",
                imageLinks.id,
                imageLinks.key);
        SetWindowText(link_preview, preview_buffer);
        EnableWindow(bsr_button, 1);
        break;
    case 1:
        sprintf(
                preview_buffer,
                "http://%s.static.itmages.ru/%s",
                imageLinks.server,
                imageLinks.full);
        SetWindowText(link_preview, preview_buffer);
        EnableWindow(bsr_button, 1);
        break;
    case 2:
        sprintf(
                preview_buffer,
                "[url=http://itmages.ru/image/view/%s/%s][img]http://%s.static.itmages.ru/%s[/img][/url]",
                imageLinks.id,
                imageLinks.key,
                imageLinks.server,
                imageLinks.thumb);
        SetWindowText(link_preview, preview_buffer);
        EnableWindow(bsr_button, 0);
        break;
    case 3:
        sprintf(
                preview_buffer,
                "<a target=\"_blank\"; href=\"http://itmages.ru/image/view/%s/%s\"><img src=\"http://%s.static.itmages.ru/%s\" /></a>",
                imageLinks.id,
                imageLinks.key,
                imageLinks.server,
                imageLinks.thumb);
        SetWindowText(link_preview, preview_buffer);
        EnableWindow(bsr_button, 0);
        break;
    }
}

void UpdateBackgroundBrush(HWND wnd)
{
    if (IsAppThemed() == TRUE) {
        g_ThemeActive = 1;
    } else {
        g_ThemeActive = 0;
    }

    if (g_StaticBrush != NULL) {
        DeleteObject(g_StaticBrush);
    }

    g_StaticBrush = NULL;

    if (g_ThemeActive == 1) {
        RECT wnd_rect;

        GetWindowRect(wnd, &wnd_rect);

        HDC wnd_dc = GetDC(wnd);
        HDC wnd_dc_mem = CreateCompatibleDC(wnd_dc);

        HBITMAP bitmap = CreateCompatibleBitmap(wnd_dc,
                wnd_rect.right - wnd_rect.left, wnd_rect.bottom - wnd_rect.top);
        HBITMAP bitmap_old = (HBITMAP) SelectObject(wnd_dc_mem, bitmap);

        SendMessage(wnd, WM_PRINTCLIENT, (WPARAM) (wnd_dc_mem),
                (LPARAM) (PRF_ERASEBKGND | PRF_CLIENT | PRF_NONCLIENT));

        g_StaticBrush = CreatePatternBrush(bitmap);

        SelectObject(wnd_dc_mem, bitmap_old);

        DeleteObject(bitmap);
        DeleteDC(wnd_dc_mem);
        ReleaseDC(wnd, wnd_dc);
    }
}

void SetThemedBackground(
        HWND wnd,
        HWND static_wnd,
        HDC static_hdc)
{
    RECT wnd_rect;

    GetWindowRect(static_wnd, &wnd_rect);
    SetBkMode(static_hdc, TRANSPARENT);
    MapWindowPoints(NULL, wnd, (LPPOINT) (&wnd_rect), 2);
    SetBrushOrgEx(static_hdc, -wnd_rect.left, -wnd_rect.top, NULL);
}

void SetSystemFont(
        HWND parent_wnd,
        ItmagesWindowIds first_wnd,
        ItmagesWindowIds last_wnd)
{
    size_t i;

    for (i = first_wnd; i <= last_wnd; ++i) {
        SendDlgItemMessage(parent_wnd, i, WM_SETFONT, (WPARAM) g_SystemFont, 0);
    }
}

void HideWindows(
        HWND parent_wnd,
        ItmagesWindowIds first_wnd,
        ItmagesWindowIds last_wnd)
{
    size_t i;
    HWND hide_wnd;

    for (i = first_wnd; i <= last_wnd; ++i) {
        hide_wnd = GetDlgItem(parent_wnd, i);
        ShowWindow(hide_wnd, SW_HIDE);
    }
}

void ShowTabWindows(
        HWND parent_wnd,
        ItmagesWindowIds first_wnd,
        ItmagesWindowIds last_wnd,
        ItmagesWindowIds first_show,
        ItmagesWindowIds last_show)
{
    size_t i;
    HWND tab_wnds;

    for (i = first_wnd; i <= last_wnd; ++i) {
        tab_wnds = GetDlgItem(parent_wnd, i);
        if (i > first_show - 1 && i < last_show + 1) {
            ShowWindow(tab_wnds, SW_SHOW);
        } else {
            ShowWindow(tab_wnds, SW_HIDE);
        }
    }
}

void GetConfigFiles(ItmagesConfig *configFiles)
{
    size_t folder_len = wcslen(ITMAGES_DIR);
    size_t config_len = wcslen(CONFIG_FILE);
    size_t cookies_len = wcslen(COOKIES_FILE);

    configFiles->config = (wchar_t*) calloc(_MAX_PATH + 1, sizeof (wchar_t));
    if (configFiles->config == NULL) {
        ErrorsHandler(ERROR_MEMALLOC);
        exit(EXIT_FAILURE);
    }
    if (SHGetFolderPathW(NULL, CSIDL_APPDATA | CSIDL_FLAG_CREATE, NULL, 0, configFiles->config) != S_OK) {
        ErrorsHandler(ERROR_GETSETTINGSDIR);
    } else {
        size_t path_len = wcslen(configFiles->config);
        wmemcpy(configFiles->config + path_len, ITMAGES_DIR, folder_len + 1);
        CreateDirectoryW(configFiles->config, NULL);
        wmemcpy(configFiles->config + path_len + folder_len, CONFIG_FILE, config_len + 1);
    }
    configFiles->cookies = (wchar_t*) calloc(_MAX_PATH + 1, sizeof (wchar_t));
    if (configFiles->cookies == NULL) {
        ErrorsHandler(ERROR_MEMALLOC);
        exit(EXIT_FAILURE);
    }
    if (SHGetFolderPathW(NULL, CSIDL_COOKIES | CSIDL_FLAG_CREATE, NULL, 0, configFiles->cookies) != S_OK) {
        ErrorsHandler(ERROR_GETCOOKIESDIR);
    } else {
        size_t path_len = wcslen(configFiles->cookies);
        wmemcpy(configFiles->cookies + path_len, ITMAGES_DIR, folder_len + 1);
        CreateDirectoryW(configFiles->cookies, NULL);
        wmemcpy(configFiles->cookies + path_len + folder_len, COOKIES_FILE, cookies_len + 1);
    }
}

char *GetInstallDir()
{
    HKEY key;
    DWORD type = REG_SZ;
    DWORD size = _MAX_PATH;

    char *install_dir = (char*) calloc(_MAX_PATH + 1, sizeof (char));
    if (install_dir == NULL) {
        ErrorsHandler(ERROR_MEMALLOC);
        exit(EXIT_FAILURE);
    }
    if (RegOpenKeyEx(HKEY_CURRENT_USER, ITMAGES_REG_KEY, 0L, KEY_ALL_ACCESS, &key) == ERROR_SUCCESS) {
        if (RegQueryValueEx(key, "", NULL, &type, (LPBYTE) install_dir, &size) != ERROR_SUCCESS) {
            return NULL;
        }
    } else {
        return NULL;
    }
    RegCloseKey(key);
    return install_dir;
}

char *GetLocaleDir()
{
    char *locale_dir = GetInstallDir();
    if (locale_dir != NULL) {
        memcpy(locale_dir + strlen(locale_dir), LOCALE_DIR, strlen(LOCALE_DIR) + 1);
    }
    return locale_dir;
}

int LoadDocFile(
        HWND load_wnd,
        char *docs_dir,
        char *load_file)
{
    char *doc_file = GetInstallDir();
    if (doc_file == NULL) {
		return -1;
	}
	
    size_t docs_len = strlen(DOCS_DIR);
    size_t load_len = strlen(load_file);

    char *buffer = (char*) malloc((docs_len + load_len + 1) * sizeof (char));
    if (buffer == NULL) {
        ErrorsHandler(ERROR_MEMALLOC);
        exit(EXIT_FAILURE);
    }
    memcpy(buffer, docs_dir, docs_len + 1);
    memcpy(buffer + strlen(buffer), load_file, load_len + 1);
    memcpy(doc_file + strlen(doc_file), buffer, docs_len + load_len + 1);
    free(buffer);

    wchar_t * doc_file_wide = AcpToWide(doc_file);
    free(doc_file);

    FILE *file;
    long file_len;

    file = _wfopen(doc_file_wide, L"rb");
    if (file == NULL) {
		free(doc_file_wide);
		return -1;
	}
    
	fseek(file, 0, SEEK_END);
    file_len = ftell(file);
    rewind(file);

    char *text = (char*) malloc((file_len + 1) * sizeof (char));
    if (text == NULL) {
        ErrorsHandler(ERROR_MEMALLOC);
        exit(EXIT_FAILURE);
    }
    fread(text, 1, file_len, file);
    free(doc_file_wide);
    fclose(file);

    SetWindowText(load_wnd, text);
    free(text);
	return 0;
}

void FreeUserInfo(ItmagesUserInfo *userInfo)
{
    if (userInfo->login != NULL) {
        free(userInfo->login);
        userInfo->login = NULL;
    }
    if (userInfo->passwd != NULL) {
        free(userInfo->passwd);
        userInfo->passwd = NULL;
    }
    if (userInfo->address != NULL) {
        free(userInfo->address);
        userInfo->address = NULL;
    }
    if (userInfo->port != NULL) {
        free(userInfo->port);
        userInfo->port = NULL;
    }
}

void FreeConfig(ItmagesConfig *configFiles)
{
    if (configFiles->config != NULL) {
        free(configFiles->config);
        configFiles->config = NULL;
    }
    if (configFiles->cookies != NULL) {
        free(configFiles->cookies);
        configFiles->cookies = NULL;
    }
}

void FreeImageData(ItmagesImageData *imageData)
{
    if (imageData->path != NULL) {
        free(imageData->path);
        imageData->path = NULL;
    }
    if (imageData->basename != NULL) {
        free(imageData->basename);
        imageData->basename = NULL;
    }
    if (imageData->ext != NULL) {
        free(imageData->ext);
        imageData->ext = NULL;
    }
}






























